#!/usr/bin/perl
#
# diststat.pl
# Distance calculater from latitude and longitude
# 

my %idokeido;
my @points;

while (<>) {
	if (/(\S+)\s+([,\.\d]+)\s+([,\.\d]+)/) {
		my $pointname = $1;
		push(@points, $pointname);
		$idokeido{$pointname}{'ido'} = $2;
		$idokeido{$pointname}{'keido'} = $3;
		if ($idokeido{$pointname}{'ido'} =~ /(\d+\.?\d*),(\d+\.?\d*),(\d+\.?\d*)/) {
			$idokeido{$pointname}{'ido'} = $1 + ($2 / 60) + ($3 / 3600);
		} elsif ($idokeido{$pointname}{'ido'} =~ /(\d+\.?\d*),(\d+\.?\d*)/) {
			$idokeido{$pointname}{'ido'} = $1 + ($2 / 60);
		}
		if ($idokeido{$pointname}{'keido'} =~ /(\d+\.?\d*),(\d+\.?\d*),(\d+\.?\d*)/) {
			$idokeido{$pointname}{'keido'} = $1 + ($2 / 60) + ($3 / 3600);
		} elsif ($idokeido{$pointname}{'keido'} =~ /(\d+\.?\d*),(\d+\.?\d*)/) {
			$idokeido{$pointname}{'keido'} = $1 + ($2 / 60);
		}
	} else {
		print("Input file is invalid.\n");
	}
}

print(",");
foreach my $pointname (@points) {
	print("$pointname,");
}
print("\n");
foreach my $start (@points) {
	print("$start,");
	foreach my $goal (@points) {
		if ($start ne $goal) {
			printf("%.4f,", &diststat($idokeido{$start}{'ido'}, $idokeido{$start}{'keido'}, $idokeido{$goal}{'ido'}, $idokeido{$goal}{'keido'}));
		} else {
			print('0,');
		}
	}
	print("\n");
}

sub diststat {
	my ($startido, $startkeido, $goalido, $goalkeido) = @_;
	my $pi = atan2(1, 1) * 4;
	my $startido = $pi * $startido / 180;
	my $startkeido = $pi * $startkeido / 180;
	my $goalido = $pi * $goalido / 180;
	my $goalkeido = $pi * $goalkeido / 180;
	my $temp1 = 6378.137 / (sqrt(1 - 0.006694470 * sin($startido) ** 2));
	my $a = $temp1 * cos($startido) * cos($startkeido);
	my $b = $temp1 * cos($startido) * sin($startkeido);
	my $c = $temp1 * (1 - 0.006694470) * sin($startido);
	my $temp2 = 6378.137 / (sqrt(1 - 0.006694470 * sin($goalido) ** 2));
	my $d = $temp2 * cos($goalido) * cos($goalkeido);
	my $e = $temp2 * cos($goalido) * sin($goalkeido);
	my $f = $temp2 * (1 - 0.006694470) * sin($goalido);
	my $dist = 6378.137 * 2 * &asin(sqrt(($a - $d) ** 2 + ($b - $e) ** 2 + ($c - $f) ** 2) / (6378.137 * 2));
	return($dist);
}

sub asin {
	return(atan2($_[0], sqrt(1 - $_[0] ** 2)));
}
